/***********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/

A Java 6 Swing application for color testing and conversions from RGB to
hexadecimal data formats and vice versa.

Copyright (C) 2008, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/
package com.woven_media.colorsafe;

import java.awt.Font;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

/**
 * PaletteInfoPanel class extends <code>javax.swing.JPanel</code> and is responsible
 * for displaying a <i>safe</i> color and the current color swatchs.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 */
public class PaletteInfoPanel extends JPanel
{
    private JLabel lblRGB, lblHex, lblName;
    private Font labelFont;
    private JPanel swatch;
    private Color currentColor;
    private String currentHex, currentName;
    private int[] currentRGB, empty;

    /**
     * Class constructor creates and adds the <i>swatch</i>
     * panels to the main panel.
     *
     * @since 1.0
     */
    PaletteInfoPanel()
    {
        int[] tmp = new int[] {-1,-1,-1};
        lblRGB = new JLabel("",JLabel.CENTER);
        lblHex = new JLabel("",JLabel.CENTER);
        lblName = new JLabel("",JLabel.CENTER);
        lblName.setPreferredSize(new Dimension(120, 10));
        lblRGB.setFont(new Font("Monospaced", Font.PLAIN, 10));
        lblHex.setFont(new Font("Monospaced", Font.PLAIN, 10));
        lblName.setFont(new Font("Monospaced", Font.PLAIN, 10));
        lblRGB.setForeground(Color.black);
        lblHex.setForeground(Color.black);
        lblName.setForeground(Color.black);

        swatch = new JPanel();
        swatch.setBorder(new LineBorder(Color.black, 1));

        add(lblRGB);
        add(swatch);
        add(lblHex);
        swatch.add(lblName);

        setLayout(new GridLayout(1,3,0,0));
        setDoubleBuffered(true);
        selectedObject(this.getBackground(), "", tmp, "NULL");
    }

    /**
     * Sets the appropriate members with the current color information.
     *
     * @param currentColor <code>java.awt.Color</code> object of this current color.
     * @param currentHex   string of the hexadecimal value for this current color.
     * @param currentRGB   <code>int</code> array of RGB values.
     * @param currentName  string for named colors if it exists.
     * @since 1.0
     */
    public void selectedObject(Color currentColor, String currentHex, int[] currentRGB, String currentName)
    {
        this.currentColor = currentColor;
        this.currentHex   = currentHex;
        this.currentRGB   = currentRGB;
        this.currentName  = currentName;
    }

    /**
     * Averages font color relative to the background color for this instance.
     * If the background is a light shade the font color is black,
     * otherwise the font color is white.
     *
     * @return <code>java.awt.Color</code> object representing this current font color.
     * @since  1.0
     */
    public Color getSmartFontColor()
    {
        Color fg;
        int avg = ((this.currentRGB[0] + this.currentRGB[1] + this.currentRGB[2]) / 3);
        if(avg > 128) // (256 / 2)
        {
            fg = Color.black;
        }
        else
        {
            fg = Color.white;
        }
        return fg;
    }

    /**
     * Sets the <code>PaletteInfoPanel</code> values to the <code>selectedObject</code> values
     * for this instance.
     *
     * @since  1.0
     */
    public void updateInfoPanel()
    {
        swatch.setBackground(this.currentColor);
        if(this.currentRGB[0] != -1)
        {
            lblHex.setText("#" + this.currentHex);
            lblRGB.setText(this.currentRGB[0] + "," + this.currentRGB[1] + "," + this.currentRGB[2]);
        }

        if(!this.currentName.equals("NULL"))
        {
            lblName.setVisible(true);
            lblName.setForeground(getSmartFontColor());
            lblName.setText(this.currentName);
        }
    }

    /**
     * Repaints background when display information changes.
     *
     * @see   javax.swing.JComponent#paintComponent
     * @see   java.awt.Graphics
     * @since 1.0
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g); //paint background
        updateInfoPanel();
    }

} // end class